﻿/* ***********************************************************************
The classes in the package
duel.bkde.* were created by

Barbara Kaskosz of the University of Rhode Island
(bkaskosz@math.uri.edu) and
Douglas E. Ensley of Shippensburg University
(DEEnsley@ship.edu).

Feel free to use and modify the classes in the package 
bkde.as3.* to create
your own Flash applications for educational purposes under the
following two conditions:

1) Please include a line acknowledging this work and the authors
with your application in a way that is visible in your compiled file
or on your html page. 

2) If you choose to modify the classes in the package 
duel.bkde.*, put them into a package different from
the original. In this case too, please acknowledge this work
in a way visible to the user.

This work is supported in part by the National Science Foundation
under the grant DUE-0535327.

Last modified: December 2, 2007
************************************************************************ */

package duel.bkde {
	

public class MatrixUtils {
	
	
	public static function pointNewView(v:Array,theta:Number,phi:Number):Array {
	
	var newCoords:Array=[];
	
	newCoords[0]=v[0]*Math.cos(theta)*Math.sin(phi)+v[1]*Math.sin(theta)*Math.sin(phi)+v[2]*Math.cos(phi);
	
	newCoords[1]=-v[0]*Math.sin(theta)+v[1]*Math.cos(theta);
	
	newCoords[2]=-v[0]*Math.cos(theta)*Math.cos(phi)-v[1]*Math.sin(theta)*Math.cos(phi)+v[2]*Math.sin(phi);
	
	return newCoords;
	
	
}

public static function crossProd(v:Array, w:Array):Array {
	  
	var C:Array = [];
	
	C[0] = v[1]*w[2]-v[2]*w[1];
	C[1] = v[2]*w[0]-v[0]*w[2];
	C[2] = v[0]*w[1]-v[1]*w[0]; 
	
	return C;
}
	
public static function dotProd(v:Array, w:Array):Number {
	  
	return v[0]*w[0]+v[1]*w[1]+v[2]*w[2];
	
}

public static function vecMinusVec(v:Array, w:Array):Array {
	  
	var C:Array = [];
	
	C[0] = v[0]-w[0];
	C[1] = v[1]-w[1];
	C[2] = v[2]-w[2];
	
	return C;
}

public static function vecPlusVec(v:Array, w:Array):Array {
	  
	var C:Array = [];
	
	C[0] = v[0]+w[0];
	C[1] = v[1]+w[1];
	C[2] = v[2]+w[2];
	
	return C;
}

public static function vecTimesNum(v:Array, a:Number):Array {
	  
	var C:Array = [];
	
	C[0] = v[0]*a;
	C[1] = v[1]*a;
	C[2] = v[2]*a;
	
	return C;
}

public static function calcMag(v:Array):Number {
	  
	return Math.sqrt(Math.pow(v[0],2)+Math.pow(v[1],2)+Math.pow(v[2],2));
	
}


public static function unitVec(v:Array):Array {
	  
	var mag:Number=Math.sqrt(Math.pow(v[0],2)+Math.pow(v[1],2)+Math.pow(v[2],2));
	
	var C:Array=[0,0,0];
	
	if(mag>0){
		
		C[0]=v[0]/mag;
		
		C[1]=v[1]/mag;
		
		C[2]=v[2]/mag;
		
	}
	
	return C;
}

public static function roundVec(v:Array):Array {
	  
	var C:Array=[];
		
		C[0]=Math.round(v[0]*100000)/100000;
		
		C[1]=Math.round(v[1]*100000)/100000;
		
		C[2]=Math.round(v[2]*100000)/100000;
		
	return C;
}

	
public static function matrixByVector(A:Array, B:Array):Array {
	  
	var C:Array = [];
	
	C[0] = A[0][0]*B[0]+A[0][1]*B[1]+A[0][2]*B[2];
	C[1] = A[1][0]*B[0]+A[1][1]*B[1]+A[1][2]*B[2];
	C[2] = A[2][0]*B[0]+A[2][1]*B[1]+A[2][2]*B[2]; 
	
	return C;
}
	
	
  public static function matrixByMatrix(A:Array, B:Array):Array {
	
	var C:Array=[];
	
	var i:Number;
	
	for(i=0;i<3;i++){
		
		C[i]=[];
		
	}
	
	C[0][0] = A[0][0]*B[0][0]+A[0][1]*B[1][0]+A[0][2]*B[2][0];
	C[0][1] = A[0][0]*B[0][1]+A[0][1]*B[1][1]+A[0][2]*B[2][1];
	C[0][2] = A[0][0]*B[0][2]+A[0][1]*B[1][2]+A[0][2]*B[2][2];
	C[1][0] = A[1][0]*B[0][0]+A[1][1]*B[1][0]+A[1][2]*B[2][0];
	C[1][1] = A[1][0]*B[0][1]+A[1][1]*B[1][1]+A[1][2]*B[2][1];
	C[1][2] = A[1][0]*B[0][2]+A[1][1]*B[1][2]+A[1][2]*B[2][2];
	C[2][0] = A[2][0]*B[0][0]+A[2][1]*B[1][0]+A[2][2]*B[2][0];
	C[2][1] = A[2][0]*B[0][1]+A[2][1]*B[1][1]+A[2][2]*B[2][1];
	C[2][2] = A[2][0]*B[0][2]+A[2][1]*B[1][2]+A[2][2]*B[2][2];
	
	return C;
	
}


public static function projectPoint(point:Array,flen:Number):Array {
	
	var projection:Array=[];
	
	projection[0]=flen/(flen-point[0])*point[1];
	
	projection[1]=-flen/(flen-point[0])*point[2];
	
	return projection;
	
}

public static function calcDist(point:Array,flen:Number):Number {
	
	var dist:Number;
	
	dist=Math.sqrt(Math.pow(point[0]-flen,2)+Math.pow(point[1],2)+Math.pow(point[2],2));
	
	return dist;
	
}

public static function rotMatrix(a:Number, b:Number, z:Number, theta:Number):Array {
	
	var axLen:Number;
	
	var T:Array=[];
	
	var radtheta:Number;
	
	var i:Number;
	
	var cosT:Number;
	
	var sinT:Number;
	
	var diffT:Number;
	
	//radtheta=Math.PI/180*theta;
	
	radtheta=theta;
	
	axLen = Math.sqrt(a*a+b*b+z*z);
	
	for(i=0;i<3;i++){
		
		T[i]=[];
		
	}
	
	T =[[1,0,0], [0,1,0], [0,0,1]];
	
	if (axLen>.0001) {
		
		a /= axLen;
		b /= axLen;
		z /= axLen;
		
		cosT = Math.cos(radtheta);
		sinT = Math.sin(radtheta);
		diffT = 1-cosT;
		T[0][0] = diffT*a*a+cosT;
		T[0][1] = diffT*a*b-sinT*z;
		T[0][2] = diffT*a*z+sinT*b;
		T[1][0] = diffT*a*b+sinT*z;
		T[1][1] = diffT*b*b+cosT;
		T[1][2] = diffT*b*z-sinT*a;
		T[2][0] = diffT*a*z-sinT*b;
		T[2][1] = diffT*b*z+sinT*a;
		T[2][2] = diffT*z*z+cosT;
			
		
	}
	
	  	return T;
}


		
}

}